package com.digiwin.athena.app.ptc.service.production;

import com.alibaba.fastjson.TypeReference;
import com.alibaba.nacos.common.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.common.enums.TabStatusEnums;
import com.digiwin.athena.app.ptc.infra.entity.ProductionDetailsEntity;
import com.digiwin.athena.app.ptc.infra.service.ProductionDetailsService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @auther: zhenggl
 * @date: 2023/9/12
 */
@Service
@Log4j2
public class ProductionTasUpdateEAIService extends AbsEAIService {

    @Resource
    private ProductionDetailsService productionDetailsService;

    @Override
    public String getServiceName() {
        return ProductionUtil.PRODUCTION_TASK_UPDATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        //根据bk更新状态为已完成
        EAIRequest eaiRequest = EAIRequest.build(messageBody);
        List<ProductionDetailsEntity> productionDetailsEntities = eaiRequest.getObject("get_data", new TypeReference<List<ProductionDetailsEntity>>() {});

        for (ProductionDetailsEntity productionDetailsEntity : productionDetailsEntities) {
            if (StringUtils.isEmpty(productionDetailsEntity.getWorkNo())){
                throw new DWBusinessException("缺少必填参数");
            }
        }

        LambdaUpdateWrapper<ProductionDetailsEntity> ump = new LambdaUpdateWrapper<>();
        ump.set(ProductionDetailsEntity::getProductionStatus, TabStatusEnums.COMPLETED.getValue().toString());
        List<String> woNoList = productionDetailsEntities.stream().map(ProductionDetailsEntity::getWorkNo).collect(Collectors.toList());
        ump.in(ProductionDetailsEntity::getWorkNo,woNoList);


        productionDetailsService.update(ump);

        return buildOK();
    }
}
